import {
  Rule, Tree, SchematicsException,
  apply, url, applyTemplates, move,
  chain, mergeWith
} from '@angular-devkit/schematics';

import { strings, normalize, experimental } from '@angular-devkit/core';

import { Schema as MyServiceSchema } from './schema';

import { getTemplate, getE2eTemplate } from './utils/workspace.utils';

export function myCustomElement(options: MyServiceSchema): Rule {
  return (tree: Tree) => {
    const workspaceConfig = tree.read('/angular.json');
    if (!workspaceConfig) {
      throw new SchematicsException('Could not find Angular workspace configuration');
    }

    // convert workspace to string
    const workspaceContent = workspaceConfig.toString();

    // parse workspace string into JSON object
    const workspace: experimental.workspace.WorkspaceSchema = JSON.parse(workspaceContent);

    if (!options.project) {
      options.project = workspace.defaultProject;
    }

    const projectName = options.project as string;

    const project = workspace.projects[projectName];

    const projectType = project.projectType === 'application' ? 'app' : 'lib';

    if (options.path === undefined) {
      options.path = `${project.sourceRoot}/${projectType}/${options.name}`;
    }

    options.style = options.style ? options.style : 'scss';

    const templateSource = apply(url('./files'), [
      applyTemplates({
        classify: strings.classify,
        dasherize: strings.dasherize,
        name: options.name
      }),
      move(normalize(options.path as string))
    ]);

    try {

      const projectTemplate = getTemplate(options.name);
      const projectE2eTemplate = getE2eTemplate(options.name);

      workspace.projects[`${options.name}`] = projectTemplate;
      workspace.projects[`${options.name}-e2e`] = projectE2eTemplate;

      if (!workspace.defaultProject) {
        workspace.defaultProject = options.name;
      }

      tree.overwrite('/angular.json', JSON.stringify(workspace, null, 2));
    } catch (e) {
      console.error(`🚫 Failed in update angular.json`);
    }

    return chain([
      mergeWith(templateSource)
    ]);
  };
}
